function [R, t, err] = GN(xs, xe, P1w, P2w, R0)

%first compute the weight of each line and the normal of the interpretation plane passing through to camera center and the line 
n = length(xs);
w  = zeros(n,1);
nc = zeros(3,n);
for i=1:n
    w(i) = 1/norm(xs(:,i)-xe(:,i)); % the weight of a line is the inverse of its image length
    temp = xcross(xs(:,i),xe(:,i));  
    nc(:,i) = temp/norm(temp);   
end

s0 = CGR(R0);

A= zeros(n*2,12);
for i= 1:n
    nx= nc(1,i); ny= nc(2,i); nz= nc(3,i);
    px= P1w(1,i); py= P1w(2,i); pz= P1w(3,i);
    A(i*2-1,:)= [nx*px ny*px nz*px nx*py ny*py nz*py nx*pz ny*pz nz*pz nx ny nz];
    px= P2w(1,i); py= P2w(2,i); pz= P2w(3,i);
    A(i*2,:)= [nx*px ny*px nz*px nx*py ny*py nz*py nx*pz ny*pz nz*pz nx ny nz];
end

A1= A(:,1:9);
A2= A(:,10:12);

A4= -inv(A2.'*A2)*A2.'*A1;
A3= A1+A2*A4;
M= A3.'*A3;

s= GaussNewton(M,s0);

R= CGR2(s);
x = R(:);
t= A4*x;

err = x' * M * x;

return

function c = xcross(a,b)

c = [a(2)*b(3)-a(3)*b(2);
     a(3)*b(1)-a(1)*b(3);
     a(1)*b(2)-a(2)*b(1)];
 
return
 
function s= CGR(R)

A= R.';

q4= sqrt(1+A(1,1)+A(2,2)+A(3,3))/2;

if q4 > 0.01
	q1= (A(3,2)-A(2,3))/q4/4;
	q2= (A(1,3)-A(3,1))/q4/4;
	q3= (A(2,1)-A(1,2))/q4/4;
else
	q1= sqrt(1+A(1,1)-A(2,2)-A(3,3))/2;
	q2= (A(1,2)+A(2,1))/q1/4;
	q3= (A(1,3)+A(3,1))/q1/4;
	q4= (A(3,2)-A(2,3))/q1/4;
end

s= [q1; q2; q3]/q4;

return

function R= CGR2(s)

s1= s(1);
s2= s(2);
s3= s(3);

R= [ s1^2 - s2^2 - s3^2 + 1,   2*s3 + 2*s1*s2,           2*s1*s3 - 2*s2;
     2*s1*s2 - 2*s3, - s1^2 + s2^2 - s3^2 + 1,           2*s1 + 2*s2*s3;
     2*s2 + 2*s1*s3,           2*s2*s3 - 2*s1, - s1^2 - s2^2 + s3^2 + 1];
 
R= R/(1+s1*s1+s2*s2+s3*s3);

return

function s= GaussNewton(M,s0)

A1= [...
4*M(1) - 4*M(37) + 4*M(41) + 4*M(45) - 4*M(5) - 4*M(73) + 4*M(77) + 4*M(81) - 4*M(9);
6*M(10) - 6*M(14) - 6*M(18) + 6*M(2) + 6*M(28) - 6*M(32) - 6*M(36) - 6*M(38) + 6*M(4) - 6*M(40) - 6*M(74) - 6*M(76);
6*M(19) - 6*M(23) - 6*M(27) + 6*M(3) - 6*M(39) - 6*M(43) + 6*M(55) - 6*M(59) - 6*M(63) + 6*M(7) - 6*M(75) - 6*M(79);
6*M(42) - 6*M(44) - 6*M(46) + 6*M(50) + 6*M(54) - 6*M(6) + 6*M(64) - 6*M(68) - 6*M(72) + 6*M(78) + 6*M(8) - 6*M(80);
8*M(11) - 4*M(1) + 8*M(13) + 8*M(29) + 8*M(31) + 4*M(37) - 4*M(41) + 4*M(5) + 4*M(81);
8*M(12) + 8*M(16) + 8*M(20) + 8*M(22) + 8*M(30) + 8*M(34) - 4*M(42) - 4*M(44) + 4*M(46) - 4*M(50) - 4*M(54) + 8*M(56) + 8*M(58) + 4*M(6) + 4*M(64) - 4*M(68) - 4*M(72) - 4*M(78) + 4*M(8) - 4*M(80);
8*M(17) - 8*M(15) + 4*M(19) - 4*M(23) - 4*M(27) + 4*M(3) - 8*M(33) + 8*M(35) - 4*M(39) + 4*M(43) - 8*M(47) - 8*M(49) - 4*M(55) + 4*M(59) + 4*M(63) + 8*M(65) + 8*M(67) - 4*M(7) - 4*M(75) + 4*M(79);
8*M(21) - 4*M(1) + 8*M(25) + 4*M(41) + 8*M(57) + 8*M(61) + 4*M(73) - 4*M(81) + 4*M(9);
4*M(14) - 4*M(10) + 4*M(18) - 4*M(2) - 8*M(24) + 8*M(26) + 4*M(28) - 4*M(32) - 4*M(36) + 4*M(38) + 4*M(4) - 4*M(40) - 8*M(48) - 8*M(52) - 8*M(60) + 8*M(62) + 8*M(66) + 8*M(70) + 4*M(74) - 4*M(76);
4*M(1) - 4*M(41) - 4*M(45) + 8*M(51) - 8*M(53) - 8*M(69) + 8*M(71) - 4*M(77) - 4*M(81);
2*M(14) - 2*M(10) - 2*M(18) - 2*M(2) - 2*M(28) + 2*M(32) - 2*M(36) + 2*M(38) - 2*M(4) + 2*M(40) - 2*M(74) - 2*M(76);
4*M(15) + 4*M(17) - 2*M(19) + 2*M(23) - 2*M(27) - 2*M(3) + 4*M(33) + 4*M(35) + 2*M(39) + 2*M(43) + 4*M(47) + 4*M(49) - 2*M(55) + 2*M(59) - 2*M(63) + 4*M(65) + 4*M(67) - 2*M(7) - 2*M(75) - 2*M(79);
4*M(12) - 4*M(16) + 4*M(20) + 4*M(22) + 4*M(30) - 4*M(34) - 2*M(42) + 2*M(44) + 2*M(46) - 2*M(50) + 2*M(54) - 4*M(56) - 4*M(58) + 2*M(6) - 2*M(64) + 2*M(68) - 2*M(72) + 2*M(78) - 2*M(8) - 2*M(80);
2*M(18) - 2*M(14) - 2*M(10) - 2*M(2) + 4*M(24) + 4*M(26) - 2*M(28) - 2*M(32) + 2*M(36) - 2*M(38) - 2*M(4) - 2*M(40) + 4*M(48) + 4*M(52) + 4*M(60) + 4*M(62) + 4*M(66) + 4*M(70) + 2*M(74) + 2*M(76);
8*M(21) - 8*M(11) + 8*M(31) - 8*M(51) - 8*M(61) + 8*M(71);
2*M(10) + 2*M(14) + 2*M(18) + 2*M(2) - 4*M(24) + 4*M(26) + 2*M(28) + 2*M(32) + 2*M(36) + 2*M(38) + 2*M(4) + 2*M(40) - 4*M(48) + 4*M(52) + 4*M(60) - 4*M(62) + 4*M(66) - 4*M(70) + 2*M(74) + 2*M(76);
2*M(27) - 2*M(23) - 2*M(19) - 2*M(3) - 2*M(39) - 2*M(43) - 2*M(55) - 2*M(59) + 2*M(63) - 2*M(7) + 2*M(75) + 2*M(79);
4*M(22) - 4*M(16) - 4*M(20) - 4*M(12) + 4*M(30) + 4*M(34) + 2*M(42) - 2*M(44) + 2*M(46) + 2*M(50) - 2*M(54) - 4*M(56) + 4*M(58) + 2*M(6) - 2*M(64) - 2*M(68) + 2*M(72) - 2*M(78) - 2*M(8) + 2*M(80);
4*M(15) - 4*M(17) + 2*M(19) + 2*M(23) + 2*M(27) + 2*M(3) - 4*M(33) + 4*M(35) + 2*M(39) + 2*M(43) + 4*M(47) - 4*M(49) + 2*M(55) + 2*M(59) + 2*M(63) - 4*M(65) + 4*M(67) + 2*M(7) + 2*M(75) + 2*M(79);
2*M(44) - 2*M(42) - 2*M(46) - 2*M(50) - 2*M(54) - 2*M(6) + 2*M(64) + 2*M(68) + 2*M(72) - 2*M(78) + 2*M(8) + 2*M(80)];

A2= [...
2*M(10) - 2*M(14) - 2*M(18) + 2*M(2) + 2*M(28) - 2*M(32) - 2*M(36) - 2*M(38) + 2*M(4) - 2*M(40) - 2*M(74) - 2*M(76);
8*M(11) - 4*M(1) + 8*M(13) + 8*M(29) + 8*M(31) + 4*M(37) - 4*M(41) + 4*M(5) + 4*M(81);
4*M(12) + 4*M(16) + 4*M(20) + 4*M(22) + 4*M(30) + 4*M(34) - 2*M(42) - 2*M(44) + 2*M(46) - 2*M(50) - 2*M(54) + 4*M(56) + 4*M(58) + 2*M(6) + 2*M(64) - 2*M(68) - 2*M(72) - 2*M(78) + 2*M(8) - 2*M(80);
4*M(17) - 4*M(15) + 2*M(19) - 2*M(23) - 2*M(27) + 2*M(3) - 4*M(33) + 4*M(35) - 2*M(39) + 2*M(43) - 4*M(47) - 4*M(49) - 2*M(55) + 2*M(59) + 2*M(63) + 4*M(65) + 4*M(67) - 2*M(7) - 2*M(75) + 2*M(79);
6*M(14) - 6*M(10) - 6*M(18) - 6*M(2) - 6*M(28) + 6*M(32) - 6*M(36) + 6*M(38) - 6*M(4) + 6*M(40) - 6*M(74) - 6*M(76);
8*M(15) + 8*M(17) - 4*M(19) + 4*M(23) - 4*M(27) - 4*M(3) + 8*M(33) + 8*M(35) + 4*M(39) + 4*M(43) + 8*M(47) + 8*M(49) - 4*M(55) + 4*M(59) - 4*M(63) + 8*M(65) + 8*M(67) - 4*M(7) - 4*M(75) - 4*M(79);
8*M(12) - 8*M(16) + 8*M(20) + 8*M(22) + 8*M(30) - 8*M(34) - 4*M(42) + 4*M(44) + 4*M(46) - 4*M(50) + 4*M(54) - 8*M(56) - 8*M(58) + 4*M(6) - 4*M(64) + 4*M(68) - 4*M(72) + 4*M(78) - 4*M(8) - 4*M(80);
2*M(18) - 2*M(14) - 2*M(10) - 2*M(2) + 4*M(24) + 4*M(26) - 2*M(28) - 2*M(32) + 2*M(36) - 2*M(38) - 2*M(4) - 2*M(40) + 4*M(48) + 4*M(52) + 4*M(60) + 4*M(62) + 4*M(66) + 4*M(70) + 2*M(74) + 2*M(76);
8*M(21) - 8*M(11) + 8*M(31) - 8*M(51) - 8*M(61) + 8*M(71);
2*M(10) + 2*M(14) + 2*M(18) + 2*M(2) - 4*M(24) + 4*M(26) + 2*M(28) + 2*M(32) + 2*M(36) + 2*M(38) + 2*M(4) + 2*M(40) - 4*M(48) + 4*M(52) + 4*M(60) - 4*M(62) + 4*M(66) - 4*M(70) + 2*M(74) + 2*M(76);
4*M(1) - 4*M(37) + 4*M(41) - 4*M(45) - 4*M(5) + 4*M(73) - 4*M(77) + 4*M(81) + 4*M(9);
6*M(42) + 6*M(44) - 6*M(46) + 6*M(50) - 6*M(54) - 6*M(6) - 6*M(64) + 6*M(68) - 6*M(72) - 6*M(78) - 6*M(8) - 6*M(80);
6*M(23) - 6*M(19) - 6*M(27) - 6*M(3) + 6*M(39) - 6*M(43) + 6*M(55) - 6*M(59) + 6*M(63) + 6*M(7) - 6*M(75) + 6*M(79);
4*M(1) - 4*M(41) + 4*M(45) + 8*M(51) + 8*M(53) + 8*M(69) + 8*M(71) + 4*M(77) - 4*M(81);
4*M(10) - 4*M(14) + 4*M(18) + 4*M(2) + 8*M(24) + 8*M(26) - 4*M(28) + 4*M(32) - 4*M(36) - 4*M(38) - 4*M(4) + 4*M(40) + 8*M(48) - 8*M(52) - 8*M(60) - 8*M(62) + 8*M(66) - 8*M(70) + 4*M(74) - 4*M(76);
8*M(21) - 4*M(1) - 8*M(25) + 4*M(41) - 8*M(57) + 8*M(61) - 4*M(73) - 4*M(81) - 4*M(9);
2*M(54) - 2*M(44) - 2*M(46) - 2*M(50) - 2*M(42) - 2*M(6) - 2*M(64) - 2*M(68) + 2*M(72) + 2*M(78) - 2*M(8) + 2*M(80);
2*M(27) - 4*M(17) - 2*M(19) - 2*M(23) - 4*M(15) - 2*M(3) + 4*M(33) + 4*M(35) - 2*M(39) + 2*M(43) - 4*M(47) + 4*M(49) + 2*M(55) + 2*M(59) - 2*M(63) - 4*M(65) + 4*M(67) + 2*M(7) + 2*M(75) - 2*M(79);
4*M(16) - 4*M(12) - 4*M(20) + 4*M(22) + 4*M(30) - 4*M(34) + 2*M(42) + 2*M(44) + 2*M(46) + 2*M(50) + 2*M(54) + 4*M(56) - 4*M(58) + 2*M(6) + 2*M(64) + 2*M(68) + 2*M(72) + 2*M(78) + 2*M(8) + 2*M(80);
2*M(19) + 2*M(23) + 2*M(27) + 2*M(3) + 2*M(39) - 2*M(43) - 2*M(55) - 2*M(59) - 2*M(63) - 2*M(7) + 2*M(75) - 2*M(79)];

A3= [...
2*M(19) - 2*M(23) - 2*M(27) + 2*M(3) - 2*M(39) - 2*M(43) + 2*M(55) - 2*M(59) - 2*M(63) + 2*M(7) - 2*M(75) - 2*M(79);
4*M(12) + 4*M(16) + 4*M(20) + 4*M(22) + 4*M(30) + 4*M(34) - 2*M(42) - 2*M(44) + 2*M(46) - 2*M(50) - 2*M(54) + 4*M(56) + 4*M(58) + 2*M(6) + 2*M(64) - 2*M(68) - 2*M(72) - 2*M(78) + 2*M(8) - 2*M(80);
8*M(21) - 4*M(1) + 8*M(25) + 4*M(41) + 8*M(57) + 8*M(61) + 4*M(73) - 4*M(81) + 4*M(9);
2*M(14) - 2*M(10) + 2*M(18) - 2*M(2) - 4*M(24) + 4*M(26) + 2*M(28) - 2*M(32) - 2*M(36) + 2*M(38) + 2*M(4) - 2*M(40) - 4*M(48) - 4*M(52) - 4*M(60) + 4*M(62) + 4*M(66) + 4*M(70) + 2*M(74) - 2*M(76);
4*M(15) + 4*M(17) - 2*M(19) + 2*M(23) - 2*M(27) - 2*M(3) + 4*M(33) + 4*M(35) + 2*M(39) + 2*M(43) + 4*M(47) + 4*M(49) - 2*M(55) + 2*M(59) - 2*M(63) + 4*M(65) + 4*M(67) - 2*M(7) - 2*M(75) - 2*M(79);
4*M(18) - 4*M(14) - 4*M(10) - 4*M(2) + 8*M(24) + 8*M(26) - 4*M(28) - 4*M(32) + 4*M(36) - 4*M(38) - 4*M(4) - 4*M(40) + 8*M(48) + 8*M(52) + 8*M(60) + 8*M(62) + 8*M(66) + 8*M(70) + 4*M(74) + 4*M(76);
8*M(21) - 8*M(11) + 8*M(31) - 8*M(51) - 8*M(61) + 8*M(71);
6*M(27) - 6*M(23) - 6*M(19) - 6*M(3) - 6*M(39) - 6*M(43) - 6*M(55) - 6*M(59) + 6*M(63) - 6*M(7) + 6*M(75) + 6*M(79);
8*M(22) - 8*M(16) - 8*M(20) - 8*M(12) + 8*M(30) + 8*M(34) + 4*M(42) - 4*M(44) + 4*M(46) + 4*M(50) - 4*M(54) - 8*M(56) + 8*M(58) + 4*M(6) - 4*M(64) - 4*M(68) + 4*M(72) - 4*M(78) - 4*M(8) + 4*M(80);
4*M(15) - 4*M(17) + 2*M(19) + 2*M(23) + 2*M(27) + 2*M(3) - 4*M(33) + 4*M(35) + 2*M(39) + 2*M(43) + 4*M(47) - 4*M(49) + 2*M(55) + 2*M(59) + 2*M(63) - 4*M(65) + 4*M(67) + 2*M(7) + 2*M(75) + 2*M(79);
2*M(42) + 2*M(44) - 2*M(46) + 2*M(50) - 2*M(54) - 2*M(6) - 2*M(64) + 2*M(68) - 2*M(72) - 2*M(78) - 2*M(8) - 2*M(80);
4*M(1) - 4*M(41) + 4*M(45) + 8*M(51) + 8*M(53) + 8*M(69) + 8*M(71) + 4*M(77) - 4*M(81);
2*M(10) - 2*M(14) + 2*M(18) + 2*M(2) + 4*M(24) + 4*M(26) - 2*M(28) + 2*M(32) - 2*M(36) - 2*M(38) - 2*M(4) + 2*M(40) + 4*M(48) - 4*M(52) - 4*M(60) - 4*M(62) + 4*M(66) - 4*M(70) + 2*M(74) - 2*M(76);
6*M(54) - 6*M(44) - 6*M(46) - 6*M(50) - 6*M(42) - 6*M(6) - 6*M(64) - 6*M(68) + 6*M(72) + 6*M(78) - 6*M(8) + 6*M(80);
4*M(27) - 8*M(17) - 4*M(19) - 4*M(23) - 8*M(15) - 4*M(3) + 8*M(33) + 8*M(35) - 4*M(39) + 4*M(43) - 8*M(47) + 8*M(49) + 4*M(55) + 4*M(59) - 4*M(63) - 8*M(65) + 8*M(67) + 4*M(7) + 4*M(75) - 4*M(79);
4*M(16) - 4*M(12) - 4*M(20) + 4*M(22) + 4*M(30) - 4*M(34) + 2*M(42) + 2*M(44) + 2*M(46) + 2*M(50) + 2*M(54) + 4*M(56) - 4*M(58) + 2*M(6) + 2*M(64) + 2*M(68) + 2*M(72) + 2*M(78) + 2*M(8) + 2*M(80);
4*M(1) + 4*M(37) + 4*M(41) - 4*M(45) + 4*M(5) - 4*M(73) - 4*M(77) + 4*M(81) - 4*M(9);
6*M(10) + 6*M(14) - 6*M(18) + 6*M(2) - 6*M(28) - 6*M(32) + 6*M(36) + 6*M(38) - 6*M(4) - 6*M(40) - 6*M(74) + 6*M(76);
8*M(11) - 4*M(1) - 8*M(13) - 8*M(29) + 8*M(31) - 4*M(37) - 4*M(41) - 4*M(5) + 4*M(81);
2*M(28) - 2*M(14) - 2*M(18) - 2*M(2) - 2*M(10) + 2*M(32) + 2*M(36) - 2*M(38) + 2*M(4) + 2*M(40) - 2*M(74) + 2*M(76)];

s= s0;

pref= 10000;

for i= 1:7
    
    s1= s(1);
    s2= s(2);
    s3= s(3);

    vecs= [s1^3 , s1^2*s2 , s1^2*s3 , s1^2 , s1*s2^2 , s1*s2*s3 , s1*s2 , s1*s3^2 , s1*s3 , s1 , s2^3 , s2^2*s3 , s2^2 , s2*s3^2 , s2*s3 , s2 , s3^3 , s3^2 , s3 , 1];

    vecs1= [ 3*s1^2, 2*s1*s2, 2*s1*s3, 2*s1, s2^2, s2*s3, s2, s3^2, s3, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0];
    vecs2= [ 0, s1^2, 0, 0, 2*s1*s2, s1*s3, s1, 0, 0, 0, 3*s2^2, 2*s2*s3, 2*s2, s3^2, s3, 1, 0, 0, 0, 0];
    vecs3= [ 0, 0, s1^2, 0, 0, s1*s2, 0, 2*s1*s3, s1, 0, 0, s2^2, 0, 2*s2*s3, s2, 0, 3*s3^2, 2*s3, 1, 0];

    f1= vecs*A1;
    f2= vecs*A2;
    f3= vecs*A3;

    curf= f1*f1+f2*f2+f3*f3;
    if curf > pref
	break;
    end

    pref= curf;

    f11= vecs1*A1;
    f12= vecs2*A1;
    f13= vecs3*A1;

    f21= vecs1*A2;
    f22= vecs2*A2;
    f23= vecs3*A2;

    f31= vecs1*A3;
    f32= vecs2*A3;
    f33= vecs3*A3;

    H= [f11 f12 f13; f21 f22 f23; f31 f32 f33];
    
    s= s- inv(H)*[f1; f2; f3];
    
end
